@@ -190,7 +190,11 @@ Satrec_sgp4init(PyObject *self, PyObject *args)
 
     // See https://www.space-track.org/documentation#tle-alpha5
     if (satnum < 100000) {
-        snprintf(satnum_str, 6, "%ld", satnum);
+        snprintf(satnum_str, 6, "%05ld", satnum);
+    } else if (satnum > 339999) {
+        PyErr_SetString(PyExc_ValueError, "satellite number cannot exceed "
+                        "339999, whose Alpha 5 encoding is 'Z9999'");
+        return 0;
     } else {
         char c = 'A' + satnum / 10000 - 10;
         if (c > 'I') c++;
@@ -228,8 +232,9 @@ Satrec_sgp4init(PyObject *self, PyObject *args)
     satrec.epochyr = y % 100;
     satrec.epochdays = whole_jd - jan0jd + fraction;
 
-    /* Return true as sgp4init does, satrec.error contains any error codes */
+    satrec.classification = 'U';  /* default to Unclassified, not '\0' */
 
+    /* Return true, like sgp4init does; check satrec.error for an error code */
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -307,9 +312,9 @@ static PyMemberDef Satrec_members[] = {
     {"epochdays", T_DOUBLE, O(epochdays), 0,
      PyDoc_STR("Day of the year of this element set's epoch (see epochyr). Not set by sgp4init().")},
     {"ndot", T_DOUBLE, O(ndot), READONLY,
-     PyDoc_STR("Ballistic Coefficient in revs/day.")},
+     PyDoc_STR("Ballistic Coefficient in radians/minute^2.")},
     {"nddot", T_DOUBLE, O(nddot), READONLY,
-     PyDoc_STR("Second Derivative of Mean Motion in revs/day^3.")},
+     PyDoc_STR("Second Derivative of Mean Motion in radians/minute^3.")},
     {"bstar", T_DOUBLE, O(bstar), READONLY,
      PyDoc_STR("Drag Term in inverse Earth radii.")},
     {"ephtype", T_INT, O(ephtype), 0,
@@ -419,15 +424,17 @@ get_intldesg(SatrecObject *self, void *closure)
 static int
 set_intldesg(SatrecObject *self, PyObject *value, void *closure)
 {
-    if (!PyUnicode_Check(value))
-        return -1;
     const char *s = PyUnicode_AsUTF8(value);
     if (!s)
         return -1;
-    strncpy(self->satrec.intldesg, s, 11);
+    strncpy(self->satrec.intldesg, s, 10);
+    self->satrec.intldesg[10] = '\0';
     return 0;
 }
 
+/* The official TLE code has switched `satnum` to a string, but to avoid
+   a breaking change we make the string available as `satnum_str` and
+   still return an integer for `satnum`. */
 static PyObject *
 get_satnum(SatrecObject *self, void *closure)
 {
@@ -444,6 +451,23 @@ get_satnum(SatrecObject *self, void *closure)
     return PyLong_FromLong(n);
 }
 
+static PyObject *
+get_satnum_str(SatrecObject *self, void *closure)
+{
+    return PyUnicode_FromString(self->satrec.satnum);
+}
+
+static int
+set_satnum_str(SatrecObject *self, PyObject *value, void *closure)
+{
+    const char *s = PyUnicode_AsUTF8(value);
+    if (!s)
+        return -1;
+    strncpy(self->satrec.satnum, s, 5);
+    self->satrec.satnum[5] = '\0';
+    return 0;
+}
+
 static PyGetSetDef Satrec_getset[] = {
     {"intldesg", (getter)get_intldesg, (setter)set_intldesg,
      PyDoc_STR("International Designator: a string of up to 7 characters"
@@ -451,7 +475,11 @@ static PyGetSetDef Satrec_getset[] = {
                " two digits for the launch year, a 3-digit launch number,"
                " and one or two letters for which piece of the launch.")},
     {"satnum", (getter)get_satnum, NULL,
-     PyDoc_STR("Satellite number, from characters 3-7 of each TLE line.")},
+     PyDoc_STR("Satellite number from characters 3-7 of each TLE line,"
+               " as an integer.")},
+    {"satnum_str", (getter)get_satnum_str, (setter)set_satnum_str,
+     PyDoc_STR("Satellite number from characters 3-7 of each TLE line,"
+               " as a string.")},
     {NULL},
 };
 
