@@ -3,6 +3,7 @@
 
 from math import (acos, asinh, atan2, copysign, cos, fabs, fmod,
                   pi, sin, sinh, sqrt, tan)
+from .functions import days2mdhms
 
 undefined = None
 
@@ -480,73 +481,6 @@ def jday(year, mon, day, hr, minute, sec):
           #  - 0.5*sgn(100.0*year + mon - 190002.5) + 0.5;
           )
 
-"""
-/* -----------------------------------------------------------------------------
-*
-*                           procedure days2mdhms
-*
-*  this procedure converts the day of the year, days, to the equivalent month
-*    day, hour, minute and second.
-*
-*  algorithm     : set up array for the number of days per month
-*                  find leap year - use 1900 because 2000 is a leap year
-*                  loop through a temp value while the value is < the days
-*                  perform int conversions to the correct day and month
-*                  convert remainder into h m s using type conversions
-*
-*  author        : david vallado                  719-573-2600    1 mar 2001
-*
-*  inputs          description                    range / units
-*    year        - year                           1900 .. 2100
-*    days        - julian day of the year         0.0  .. 366.0
-*
-*  outputs       :
-*    mon         - month                          1 .. 12
-*    day         - day                            1 .. 28,29,30,31
-*    hr          - hour                           0 .. 23
-*    min         - minute                         0 .. 59
-*    sec         - second                         0.0 .. 59.999
-*
-*  locals        :
-*    dayofyr     - day of year
-*    temp        - temporary extended values
-*    inttemp     - temporary int value
-*    i           - index
-*    lmonth[12]  - int array containing the number of days per month
-*
-*  coupling      :
-*    none.
-* --------------------------------------------------------------------------- */
-"""
-
-def days2mdhms(year, days):
-
-     lmonth = (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
-
-     dayofyr = int(days // 1.0);
-     #  ----------------- find month and day of month ----------------
-     if (year % 4) == 0:
-       lmonth = (31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
-
-     i = 1;
-     inttemp = 0;
-     while dayofyr > inttemp + lmonth[i-1] and i < 12:
-
-       inttemp = inttemp + lmonth[i-1];
-       i += 1;
-
-     mon = i;
-     day = dayofyr - inttemp;
-
-     #  ----------------- find hours minutes and seconds -------------
-     temp = (days - dayofyr) * 24.0;
-     hr   = int(temp // 1.0);
-     temp = (temp - hr) * 60.0;
-     minute  = int(temp // 1.0);
-     sec  = (temp - minute) * 60.0;
-
-     return mon, day, hr, minute, sec
-
 """
 /* -----------------------------------------------------------------------------
 *
@@ -607,6 +541,6 @@ def invjday(jd):
          days    = temp - ((year - 1900) * 365.0 + leapyrs);
 
      #  ----------------- find remaing data  -------------------------
-     mon, day, hr, minute, sec = days2mdhms(year, days);
+     mon, day, hr, minute, sec = days2mdhms(year, days, None);
      sec = sec - 0.00000086400;
      return year, mon, day, hr, minute, sec
