@@ -6,7 +6,6 @@ except:
     from unittest import TestCase, main
 
 import datetime as dt
-import platform
 import re
 import os
 import sys
@@ -19,12 +18,14 @@ try:
 except ImportError:
     from StringIO import StringIO
 
-from sgp4.api import WGS72OLD, WGS72, WGS84, Satrec, jday, accelerated
+import numpy as np
+
+from sgp4.api import WGS72OLD, WGS72, WGS84, Satrec, jday
 from sgp4.earth_gravity import wgs72
 from sgp4.ext import invjday, newtonnu, rv2coe
 from sgp4.functions import days2mdhms, _day_of_year_to_month_day
 from sgp4.propagation import sgp4, sgp4init
-from sgp4 import conveniences, io, omm
+from sgp4 import api, conveniences, io, omm
 from sgp4.exporter import export_omm, export_tle
 import sgp4.model as model
 
@@ -44,10 +45,14 @@ BAD2  = '2 00007  34.2682 348.7242 1859667 331.7664  19.3264 10.82419157413669'
 VANGUARD_ATTRS = {
     # Identity
     'satnum': 5,
+    'satnum_str': '00005',
+    'classification': 'U',
     'operationmode': 'i',
     # Time
     'epochyr': 0,
+    'epochdays': 179.78495062,
     'jdsatepoch': 2451722.5,
+    'jdsatepochF': 0.78495062,
     # Orbit
     'bstar': 2.8098e-05,
     'ndot': 6.96919666594958e-13,
@@ -59,7 +64,7 @@ VANGUARD_ATTRS = {
     'no_kozai': 0.04722944544077857,
     'nodeo': 6.08638547138321,
 }
-VANGUARD_EPOCH = 18441.7849506199999894
+VANGUARD_EPOCH = 18441.78495062
 
 # Handle deprecated assertRaisesRegexp, but allow its use Python 2.6 and 2.7
 if sys.version_info[:2] == (2, 7) or sys.version_info[:2] == (2, 6):
@@ -78,7 +83,6 @@ def test_legacy_built_with_twoline2rv():
     verify_vanguard_1(sat, legacy=True)
 
 def test_satrec_initialized_with_sgp4init():
-    # epochyr and epochdays are not set by sgp4init
     sat = Satrec()
     sat.sgp4init(
         WGS72,
@@ -108,6 +112,21 @@ def test_legacy_initialized_with_sgp4init():
     )
     verify_vanguard_1(sat, legacy=True)
 
+# ------------------------------------------------------------------------
+#                            Test array API
+
+def test_whether_array_logic_writes_nan_values_to_correct_row():
+    # https://github.com/brandon-rhodes/python-sgp4/issues/87
+    l1 = "1 44160U 19006AX  20162.79712247 +.00816806 +19088-3 +34711-2 0  9997"
+    l2 = "2 44160 095.2472 272.0808 0216413 032.6694 328.7739 15.58006382062511"
+    sat = Satrec.twoline2rv(l1, l2)
+    jd0 = np.array([2459054.5, 2459055.5])
+    jd1 = np.array([0.79712247, 0.79712247])
+    e, r, v = sat.sgp4_array(jd0, jd1)
+    assert list(e) == [6, 1]
+    assert np.isnan(r).tolist() == [[False, False, False], [True, True, True]]
+    assert np.isnan(v).tolist() == [[False, False, False], [True, True, True]]
+
 # ------------------------------------------------------------------------
 #                 Other Officially Supported Routines
 #
@@ -202,15 +221,15 @@ def test_tle_export():
         satrec_old = io.twoline2rv(line1, line2, wgs72)
 
         # Generate TLE from satrec
-        out_line1, out_line2 = export_tle(satrec)
-        out_line1_old, out_line2_old = export_tle(satrec_old)
+        actual_line1, actual_line2 = export_tle(satrec)
+        actual_line1_old, actual_line2_old = export_tle(satrec_old)
 
         if satrec.satnum not in expected_errs_line1:
-            assertEqual(out_line1, line1)
-            assertEqual(out_line1_old, line1)
+            assertEqual(actual_line1, line1)
+            assertEqual(actual_line1_old, line1)
         if satrec.satnum not in expected_errs_line2:
-            assertEqual(out_line2, line2)
-            assertEqual(out_line2_old, line2)
+            assertEqual(actual_line2, line2)
+            assertEqual(actual_line2_old, line2)
 
 def test_export_tle_raises_error_for_out_of_range_angles():
     # See https://github.com/brandon-rhodes/python-sgp4/issues/70
@@ -224,6 +243,16 @@ def test_export_tle_raises_error_for_out_of_range_angles():
         )
         assertRaises(ValueError, export_tle, sat)
 
+def test_tle_import_export_round_trips():
+    for line1, line2 in [(
+        '1 44542U 19061A   21180.78220369 -.00000015  00000-0 -66561+1 0  9997',
+        '2 44542  54.7025 244.1098 0007981 318.8601 283.5781  1.86231125 12011',
+    )]:
+        sat = Satrec.twoline2rv(line1, line2)
+        outline1, outline2 = export_tle(sat)
+        assertEqual(line1, outline1)
+        assertEqual(line2, outline2)
+
 def test_all_three_gravity_models_with_twoline2rv():
     # The numbers below are those produced by Vallado's C++ code.
     # (Why does the Python version not produce the same values to
@@ -255,15 +284,11 @@ def test_all_three_gravity_models_with_sgp4init():
 
 GRAVITY_DIGITS = (
     # Why don't Python and C agree more closely?
-    4 if not accelerated
+    4 if not api.accelerated
 
-    # Insist on very high precision on my Linux laptop, to signal me if
-    # some future adjustment subtlely changes the library's results.
-    else 12 if platform.system() == 'Linux' and platform.machine() == 'x86_64'
-
-    # Otherwise, reduce our expectations.  Note that at least 6 digits
-    # past the decimal point are necessary to let the test distinguish
-    # between WSG72OLD and WGS72.  See:
+    # Otherwise, try 10 digits.  Note that at least 6 digits past the
+    # decimal point are necessary to let the test distinguish between
+    # WSG72OLD and WGS72.  See:
     # https://github.com/conda-forge/sgp4-feedstock/pull/19
     # https://github.com/brandon-rhodes/python-sgp4/issues/69
     else 10
@@ -293,10 +318,13 @@ def assert_wgs84(sat):
 
 def test_satnum_leading_spaces():
     # https://github.com/brandon-rhodes/python-sgp4/issues/81
+    # https://github.com/brandon-rhodes/python-sgp4/issues/90
     l1 = '1  4859U 21001A   21007.63955392  .00000000  00000+0  00000+0 0  9990'
     l2 = '2  4859 000.0000 000.0000 0000000 000.0000 000.0000 01.00000000    09'
     sat = Satrec.twoline2rv(l1, l2)
     assertEqual(sat.satnum, 4859)
+    assertEqual(sat.classification, 'U')
+    assertEqual(sat.intldesg, '21001A')
 
 def test_satnum_alpha5_encoding():
     def make_sat(satnum_string):
@@ -313,12 +341,24 @@ def test_satnum_alpha5_encoding():
 
     for satnum, satnum_string in cases:
         sat = make_sat(satnum_string)
-        assert sat.satnum == satnum
+        assertEqual(sat.satnum, satnum)
+        assertEqual(sat.satnum_str, satnum_string)
 
     args = sgp4init_args(VANGUARD_ATTRS)
     for satnum, satnum_string in cases:
         sat.sgp4init(WGS72, 'i', satnum, VANGUARD_EPOCH, *args)
-        assert sat.satnum == satnum
+        assertEqual(sat.satnum, satnum)
+
+def test_satnum_that_is_too_large():
+    sat = Satrec()
+    with assertRaisesRegex(ValueError, 'cannot exceed 339999'):
+        sat.sgp4init(
+            WGS72,
+            'i',
+            340000,
+            VANGUARD_EPOCH,
+            *sgp4init_args(VANGUARD_ATTRS)
+        )
 
 def test_intldesg_with_6_characters():
     sat = Satrec.twoline2rv(LINE1, LINE2)
@@ -333,14 +373,25 @@ def test_intldesg_with_7_characters():
     )
     assertEqual(sat.intldesg, '13066AE')
 
+def test_1990s_satrec_initialized_with_sgp4init():
+    sat = Satrec()
+    sat.sgp4init(
+        WGS72,
+        'i',
+        VANGUARD_ATTRS['satnum'],
+        VANGUARD_EPOCH - 365.0,  # change year 2000 to 1999
+        *sgp4init_args(VANGUARD_ATTRS)
+    )
+    assertEqual(sat.epochyr, 99)
+
 def test_setters():
     sat = Satrec()
 
     sat.classification = 'S'
     assert sat.classification == 'S'
 
-    sat.intldesg = 'Russian'
-    assert sat.intldesg == 'Russian'
+    sat.intldesg = 'abcdefg'
+    assert sat.intldesg == 'abcdefg'
 
     sat.ephtype = 23
     assert sat.ephtype == 23
@@ -351,6 +402,9 @@ def test_setters():
     sat.revnum = 1234
     assert sat.revnum == 1234
 
+    sat.satnum_str = 'abcde'
+    assert sat.satnum_str == 'abcde'
+
 def test_hyperbolic_orbit():
     # Exercise the newtonnu() code path with asinh() to see whether
     # we can replace it with the one from Python's math module.
@@ -419,6 +473,24 @@ def test_december_32():
     sat = Satrec.twoline2rv(a, b)
     assertEqual(conveniences.sat_epoch_datetime(sat), correct_epoch)
 
+def test_non_ascii_first_line():
+    if sys.version_info < (3,):
+        return
+    with assertRaisesRegex(ValueError, re.escape("""your TLE lines are broken because they contain non-ASCII characters:
+
+1 00005U 58002B   00179.78495062  .00000023\\xa0 00000-0  28098-4 0  4753
+2 00005  34.2682 348.7242 1859667 331.7664  19.3264 10.82419157413667""")):
+        io.twoline2rv(LINE1.replace('23 ', '23\xa0'), LINE2, wgs72)
+
+def test_non_ascii_second_line():
+    if sys.version_info < (3,):
+        return
+    with assertRaisesRegex(ValueError, re.escape("""your TLE lines are broken because they contain non-ASCII characters:
+
+1 00005U 58002B   00179.78495062  .00000023  00000-0  28098-4 0  4753
+2 00005 \\xa034.2682\\xa0348.7242 1859667 331.7664  19.3264 10.82419157413667\
+""")):
+        io.twoline2rv(LINE1, LINE2.replace(' 34', '\xa034'), wgs72)
 
 def test_bad_first_line():
     with assertRaisesRegex(ValueError, re.escape("""TLE format error
@@ -459,7 +531,9 @@ def verify_vanguard_1(sat, legacy=False):
     if legacy:
         attrs = attrs.copy()
         del attrs['epochyr']
+        del attrs['epochdays']
         del attrs['jdsatepoch']
+        del attrs['jdsatepochF']
 
     for name, value in attrs.items():
         try:
@@ -469,10 +543,6 @@ def verify_vanguard_1(sat, legacy=False):
             e.args = ('for attribute %s, %s' % (name, message),)
             raise e
 
-    if not legacy:
-        assertAlmostEqual(sat.epochdays, 179.78495062, delta=3e-14)
-        assertAlmostEqual(sat.jdsatepochF, 0.78495062, delta=1e-13)
-
 def sgp4init_args(d):
     """Given a dict of orbital parameters, return them in sgp4init order."""
     return (d['bstar'], d['ndot'], d['nddot'], d['ecco'], d['argpo'],
@@ -493,6 +563,7 @@ def test_satrec_against_tcppver_using_julian_dates():
         jd = satrec.jdsatepoch + whole
         fr = satrec.jdsatepochF + fraction
         e, r, v = satrec.sgp4(jd, fr)
+        assert e == satrec.error
         return e, r, v
 
     run_satellite_against_tcppver(Satrec.twoline2rv, invoke, [1,1,6,6,4,3,6])
@@ -501,6 +572,7 @@ def test_satrec_against_tcppver_using_tsince():
 
     def invoke(satrec, tsince):
         e, r, v = satrec.sgp4_tsince(tsince)
+        assert e == satrec.error
         return e, r, v
 
     run_satellite_against_tcppver(Satrec.twoline2rv, invoke, [1,1,6,6,4,3,6])
@@ -536,6 +608,7 @@ def run_satellite_against_tcppver(twoline2rv, invoke, expected_errors):
     # output in tcppver.out.
 
     data = get_data(__name__, 'tcppver.out')
+    data = data.replace(b'\r', b'')
     tcppver_lines = data.decode('ascii').splitlines(True)
 
     error_list = []
@@ -686,59 +759,61 @@ VANGUARD 1              \n\
 2 00005  34.2443 225.5254 1845686 162.2516 205.2356 10.84869164218149
 """
 
-# https://celestrak.com/NORAD/elements/gp.php?CATNR=00005&FORMAT=XML
-VANGUARD_XML = """\
+# The MARIO satellite was chosen by reading through stations.txt looking
+# for an element set with every field nonzero, since a zero in any field
+# would hide errors in our unit conversion: zero times any conversion
+# factor is zero.
+MARIO_TLE = """\
+MARIO                   \n\
+1 55123U 98067UQ  23115.44827133  .00787702  29408-3  15680-2 0  9999
+2 55123  51.6242 216.2930 0014649 331.8976  28.1241 15.99081912 18396
+"""
+
+# https://celestrak.org/NORAD/elements/gp.php?CATNR=55123&FORMAT=XML
+MARIO_XML = """\
 <?xml version="1.0" encoding="UTF-8"?>
-<ndm xmlns:xsi="https://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="https://sanaregistry.org/r/ndmxml/ndmxml-1.0-master.xsd">
+<ndm xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="https://sanaregistry.org/r/ndmxml_unqualified/ndmxml-2.0.0-master-2.0.xsd">
 <omm id="CCSDS_OMM_VERS" version="2.0">
-<header><CREATION_DATE/><ORIGINATOR/></header><body><segment><metadata><OBJECT_NAME>VANGUARD 1</OBJECT_NAME><OBJECT_ID>1958-002B</OBJECT_ID><CENTER_NAME>EARTH</CENTER_NAME><REF_FRAME>TEME</REF_FRAME><TIME_SYSTEM>UTC</TIME_SYSTEM><MEAN_ELEMENT_THEORY>SGP4</MEAN_ELEMENT_THEORY></metadata><data><meanElements><EPOCH>2020-10-13T04:52:48.472320</EPOCH><MEAN_MOTION>10.84869164</MEAN_MOTION><ECCENTRICITY>.1845686</ECCENTRICITY><INCLINATION>34.2443</INCLINATION><RA_OF_ASC_NODE>225.5254</RA_OF_ASC_NODE><ARG_OF_PERICENTER>162.2516</ARG_OF_PERICENTER><MEAN_ANOMALY>205.2356</MEAN_ANOMALY></meanElements><tleParameters><EPHEMERIS_TYPE>0</EPHEMERIS_TYPE><CLASSIFICATION_TYPE>U</CLASSIFICATION_TYPE><NORAD_CAT_ID>5</NORAD_CAT_ID><ELEMENT_SET_NO>999</ELEMENT_SET_NO><REV_AT_EPOCH>21814</REV_AT_EPOCH><BSTAR>-.22483E-4</BSTAR><MEAN_MOTION_DOT>-1.6E-7</MEAN_MOTION_DOT><MEAN_MOTION_DDOT>0</MEAN_MOTION_DDOT></tleParameters></data></segment></body></omm>
+<header><CREATION_DATE/><ORIGINATOR/></header><body><segment><metadata><OBJECT_NAME>MARIO</OBJECT_NAME><OBJECT_ID>1998-067UQ</OBJECT_ID><CENTER_NAME>EARTH</CENTER_NAME><REF_FRAME>TEME</REF_FRAME><TIME_SYSTEM>UTC</TIME_SYSTEM><MEAN_ELEMENT_THEORY>SGP4</MEAN_ELEMENT_THEORY></metadata><data><meanElements><EPOCH>2023-04-25T10:45:30.642912</EPOCH><MEAN_MOTION>15.99081912</MEAN_MOTION><ECCENTRICITY>.0014649</ECCENTRICITY><INCLINATION>51.6242</INCLINATION><RA_OF_ASC_NODE>216.2930</RA_OF_ASC_NODE><ARG_OF_PERICENTER>331.8976</ARG_OF_PERICENTER><MEAN_ANOMALY>28.1241</MEAN_ANOMALY></meanElements><tleParameters><EPHEMERIS_TYPE>0</EPHEMERIS_TYPE><CLASSIFICATION_TYPE>U</CLASSIFICATION_TYPE><NORAD_CAT_ID>55123</NORAD_CAT_ID><ELEMENT_SET_NO>999</ELEMENT_SET_NO><REV_AT_EPOCH>1839</REV_AT_EPOCH><BSTAR>.1568E-2</BSTAR><MEAN_MOTION_DOT>.787702E-2</MEAN_MOTION_DOT><MEAN_MOTION_DDOT>.29408E-3</MEAN_MOTION_DDOT></tleParameters></data></segment></body></omm>
 </ndm>
 """
 
-# https://celestrak.com/NORAD/elements/gp.php?CATNR=00005&FORMAT=CSV
-VANGUARD_CSV = """\
+# https://celestrak.com/NORAD/elements/gp.php?CATNR=55123&FORMAT=CSV
+MARIO_CSV = """\
 OBJECT_NAME,OBJECT_ID,EPOCH,MEAN_MOTION,ECCENTRICITY,INCLINATION,RA_OF_ASC_NODE,ARG_OF_PERICENTER,MEAN_ANOMALY,EPHEMERIS_TYPE,CLASSIFICATION_TYPE,NORAD_CAT_ID,ELEMENT_SET_NO,REV_AT_EPOCH,BSTAR,MEAN_MOTION_DOT,MEAN_MOTION_DDOT
-VANGUARD 1,1958-002B,2020-10-13T04:52:48.472320,10.84869164,.1845686,34.2443,225.5254,162.2516,205.2356,0,U,5,999,21814,-.22483E-4,-1.6E-7,0
+MARIO,1998-067UQ,2023-04-25T10:45:30.642912,15.99081912,.0014649,51.6242,216.2930,331.8976,28.1241,0,U,55123,999,1839,.1568E-2,.787702E-2,.29408E-3
 """
 
 def test_omm_xml_matches_old_tle():
-    line0, line1, line2 = VANGUARD_TLE.splitlines()
+    line0, line1, line2 = MARIO_TLE.splitlines()
     sat1 = Satrec.twoline2rv(line1, line2)
 
-    fields = next(omm.parse_xml(StringIO(VANGUARD_XML)))
+    fields = next(omm.parse_xml(StringIO(MARIO_XML)))
     sat2 = Satrec()
     omm.initialize(sat2, fields)
 
     assert_satellites_match(sat1, sat2)
 
 def test_omm_csv_matches_old_tle():
-    line0, line1, line2 = VANGUARD_TLE.splitlines()
+    line0, line1, line2 = MARIO_TLE.splitlines()
     sat1 = Satrec.twoline2rv(line1, line2)
 
-    fields = next(omm.parse_csv(StringIO(VANGUARD_CSV)))
+    fields = next(omm.parse_csv(StringIO(MARIO_CSV)))
     sat2 = Satrec()
     omm.initialize(sat2, fields)
 
     assert_satellites_match(sat1, sat2)
 
 def assert_satellites_match(sat1, sat2):
-    julian_fractions = {'epochdays', 'jdsatepochF'}
-    todo = {'whichconst'}
-
     for attr in dir(sat1):
         if attr.startswith('_'):
             continue
-        if attr in todo:
-            continue
         value1 = getattr(sat1, attr, None)
         if value1 is None:
             continue
         if callable(value1):
             continue
         value2 = getattr(sat2, attr)
-        if attr in julian_fractions:
-            value1 = round(value1, 10)
-            value2 = round(value2, 10)
         assertEqual(value1, value2, '%s %r != %r' % (attr, value1, value2))
 
 # Live example of OMM:
@@ -785,13 +860,16 @@ def load_tests(loader, tests, ignore):
     # breaks the doctest, so we only run the doctest on later versions.
     if sys.version_info >= (2, 7):
 
-        def setCwd(suite):
+        def setUp(suite):
             suite.olddir = os.getcwd()
             os.chdir(os.path.dirname(__file__))
-        def restoreCwd(suite):
+            suite.oldaccel = api.accelerated
+            api.accelerated = True  # so doctest passes under 2.7
+        def tearDown(suite):
             os.chdir(suite.olddir)
+            api.accelerated = suite.oldaccel
 
-        options = dict(optionflags=ELLIPSIS, setUp=setCwd, tearDown=restoreCwd)
+        options = dict(optionflags=ELLIPSIS, setUp=setUp, tearDown=tearDown)
         tests.addTests(DocTestSuite('sgp4', **options))
         tests.addTests(DocTestSuite('sgp4.conveniences', **options))
         tests.addTests(DocTestSuite('sgp4.functions', **options))
